@@ -77,7 +77,8 @@
  * are '\0' terminated strings representing decimal numbers.
  */
 
-#include <stdio.h>              /* FILE, fopen(), fread(), fwrite(), fclose() */
+#include <sys/mman.h>           /* mmap(), PROT_READ, MAP_PRIVATE, ... */
+#include <stdio.h>              /* FILE, fopen(), fread(), fwrite(), ... */
 #include <string.h>             /* strlen(), strrchr() */
 #include <errno.h>              /* errno */
 #include <ctype.h>              /* isdigit() */
@@ -85,6 +86,8 @@
 
 #include "gliv.h"
 #include "collection.h"
+#include "math_floats.h"        /* log10f() */
+#include "large_files.h"
 #include "messages.h"
 #include "tree.h"
 #include "thumbnails.h"
@@ -103,24 +106,69 @@
 
 extern GlivImage *current_image;
 
-/* Let the user choose a filename using a file selection dialog. */
-static const gchar *get_a_file(const gchar * title)
+static gchar *get_collection_source(void)
 {
-    GtkFileSelection *dialog;
+    GtkFileChooser *chooser;
     gint response;
-    const gchar *filename;
+    const gchar *label = _("Choose a collection to load");
+    gchar *filename = NULL;
 
-    dialog = GTK_FILE_SELECTION(gtk_file_selection_new(title));
+    chooser = GTK_FILE_CHOOSER(gtk_file_chooser_dialog_new(label,
+                                                           get_current_window(),
+                                                           GTK_FILE_CHOOSER_ACTION_OPEN,
+                                                           GTK_STOCK_CANCEL,
+                                                           GTK_RESPONSE_CANCEL,
+                                                           GTK_STOCK_OPEN,
+                                                           GTK_RESPONSE_ACCEPT,
+                                                           NULL));
 
-    gtk_file_selection_set_select_multiple(dialog, TRUE);
-    response = run_modal_dialog(GTK_DIALOG(dialog));
+    response = run_modal_dialog(GTK_DIALOG(chooser));
 
-    if (response == GTK_RESPONSE_OK)
-        filename = gtk_file_selection_get_filename(dialog);
-    else
-        filename = NULL;
+    if (response == GTK_RESPONSE_ACCEPT)
+        filename = gtk_file_chooser_get_filename(chooser);
 
-    gtk_widget_destroy(GTK_WIDGET(dialog));
+    gtk_widget_destroy(GTK_WIDGET(chooser));
+    return filename;
+}
+
+static gchar *get_collection_destination(void)
+{
+    GtkFileChooser *chooser;
+    gint response;
+    const gchar *label = _("Choose a file to save the collection");
+    gchar *filename = NULL;
+
+    chooser = GTK_FILE_CHOOSER(gtk_file_chooser_dialog_new(label,
+                                                           get_current_window(),
+                                                           GTK_FILE_CHOOSER_ACTION_SAVE,
+                                                           GTK_STOCK_CANCEL,
+                                                           GTK_RESPONSE_CANCEL,
+                                                           GTK_STOCK_SAVE,
+                                                           GTK_RESPONSE_ACCEPT,
+                                                           NULL));
+
+    response = run_modal_dialog(GTK_DIALOG(chooser));
+
+    if (response == GTK_RESPONSE_ACCEPT) {
+        filename = gtk_file_chooser_get_filename(chooser);
+        if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
+            GtkMessageDialog *overwrite =
+                GTK_MESSAGE_DIALOG(gtk_message_dialog_new
+                                   (get_current_window(), GTK_DIALOG_MODAL,
+                                    GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
+                                    _("Overwrite \"%s\" ?"), filename));
+
+	    response = run_modal_dialog(GTK_DIALOG(overwrite));
+	    if (response != GTK_RESPONSE_YES) {
+		g_free(filename);
+		filename = NULL;
+	    }
+
+	    gtk_widget_destroy(GTK_WIDGET(overwrite));
+        }
+    }
+
+    gtk_widget_destroy(GTK_WIDGET(chooser));
     return filename;
 }
 
@@ -389,6 +437,34 @@ static gboolean print_footer(FILE * file)
     return TRUE;
 }
 
+/*
+ * We will write the value number, then a '\0' and then the image
+ * data.  The data should be aligned if we read it via mmap, as
+ * GdkPixbuf uses a rowstride value so that every line beginning are
+ * on a 32 bit boundary.
+ * The alignment is made by padding some '0'.
+ */
+static gboolean align_next(FILE * file, gint value)
+{
+    gint length, pos, align;
+
+    /* For the (extremly ...) rare case value == 0 */
+    value += !value;
+
+    length = (int) log10f(value) + 2;
+    pos = (int) ftell(file) + length;   /* We don't need 64 bit precision */
+    align = (4 - pos % 4) % 4;  /* We align on 32 bit boundaries */
+
+    while (align > 0) {
+        if (putc('0', file) < 0)
+            return FALSE;
+
+        align--;
+    }
+
+    return TRUE;
+}
+
 /* Write the MaybeThumb part of an entry. */
 static gboolean write_thumb(tree_item * item, FILE * file)
 {
@@ -415,6 +491,10 @@ static gboolean write_thumb(tree_item * item, FILE * file)
     WRITE_NB(file, gdk_pixbuf_get_width(thumb));        /* ThumbWidth */
     WRITE_NB(file, height);     /* ThumbHeight */
     WRITE_NB(file, rowstride);  /* RowStride */
+
+    if (align_next(file, len) == FALSE)
+        return FALSE;
+
     WRITE_NB(file, len);        /* PixelDataLength */
 
     /* PixelData */
@@ -479,7 +559,7 @@ gboolean serialize_collection(void)
 {
     FILE *file;
     GNode *tree;
-    const gchar *filename;
+    gchar *filename;
     serialization_data *work;
     gboolean res;
 
@@ -487,12 +567,15 @@ gboolean serialize_collection(void)
     if (tree == NULL)
         return FALSE;
 
-    filename = get_a_file(_("Choose a file to save the collection"));
+    filename = get_collection_destination();
     if (filename == NULL) {
         end_using_tree();
         return FALSE;
     }
 
+    /* We remove the file, since we may have mmapped it */
+    remove(filename);
+
     file = fopen(filename, "w");
     if (file == NULL) {
         FILE_ERROR(filename);
@@ -541,54 +624,116 @@ gboolean serialize_collection(void)
     g_free(work);
     end_using_tree();
     destroy_progress_dialog();
+    g_free(filename);
     return FALSE;
 }
 
 /*** Deserialization ***/
 
+struct coll_src {
+    gboolean is_file;
+    union {
+        FILE *file;
+        struct {
+            guchar *base;
+            guchar *ptr;
+            guchar *end;
+        };
+    };
+};
+
+static void free_buffer(struct coll_src *source, guchar * buffer)
+{
+    if (source->is_file)
+        g_free(buffer);
+}
+
+static guchar *read_buffer(struct coll_src *source, size_t length,
+                           gboolean is_string)
+{
+    guchar *data;
+
+    if (source->is_file) {
+        data = g_new(guchar, length);
+        if (fread(data, 1, length, source->file) != length) {
+            g_free(data);
+            return NULL;
+        }
+    } else {
+        if (source->ptr + length > source->end)
+            return NULL;
+
+        data = source->ptr;
+        source->ptr += length;
+    }
+
+    if (is_string && data[length - 1] != '\0') {
+        free_buffer(source, data);
+        data = NULL;
+    }
+
+    return data;
+}
+
+static gint read_char(struct coll_src *source)
+{
+    if (source->is_file)
+        return fgetc(source->file);
+
+    if (source->ptr >= source->end)
+        return EOF;
+
+    return *(source->ptr++);
+}
+
+static GdkPixbufDestroyNotify destroy_func(struct coll_src *source)
+{
+    return (GdkPixbufDestroyNotify) (source->is_file ? g_free : NULL);
+}
+
 /* Checks that file starts with magic. */
-static gboolean check_magic(const gchar * magic, FILE * file)
+static gboolean check_magic(struct coll_src *source, const gchar * magic)
 {
     gboolean res;
     gint size = strlen(magic) + 1;
-    gchar *buffer = g_new(gchar, size);
+    guchar *buffer = read_buffer(source, size, TRUE);
 
-    if (fread(buffer, 1, size, file) != size) {
-        g_free(buffer);
+    if (buffer == NULL)
         return FALSE;
-    }
 
-    res = buffer[size - 1] == '\0' && g_str_equal(magic, buffer);
-    g_free(buffer);
+    res = g_str_equal(magic, buffer);
+    free_buffer(source, buffer);
 
     return res;
 }
 
+
+
 /* Reads a number represented by a string. -1 => ERROR */
-static gint read_number(FILE * file)
+static gint read_number(struct coll_src *source)
 {
     gint number = 0;
-    gint read_char = fgetc(file);
+    gint next_char = read_char(source);
 
-    while (isdigit(read_char)) {
-        number = number * 10 + read_char - '0';
-        read_char = fgetc(file);
+    while (isdigit(next_char)) {
+        number = number * 10 + next_char - '0';
+        next_char = read_char(source);
     }
 
-    if (read_char == '\0')
+    if (next_char == '\0')
         return number;
 
     return -1;
 }
 
-#define READ_THUMB_NB(file, var) \
-    do {                         \
-        var = read_number(file); \
-        if (var < 0)             \
-            return FALSE;        \
+#define READ_THUMB_NB(source, var) \
+    do {                           \
+        var = read_number(source); \
+        if (var < 0)               \
+            return FALSE;          \
     } while (0)
 
-static gboolean read_pixbuf(FILE * file, GdkPixbuf ** thumb)
+static gboolean read_pixbuf(struct coll_src *source, GdkPixbuf ** thumb)
 {
     GdkColorspace colorspace;
     gboolean has_alpha;
@@ -596,92 +741,84 @@ static gboolean read_pixbuf(FILE * file, GdkPixbuf ** thumb)
     gint pixel_length;
     guchar *pixel;
 
-    READ_THUMB_NB(file, colorspace);
-    READ_THUMB_NB(file, has_alpha);
+    READ_THUMB_NB(source, colorspace);
+    READ_THUMB_NB(source, has_alpha);
     if (has_alpha != FALSE && has_alpha != TRUE)
         return FALSE;
 
-    READ_THUMB_NB(file, bits_per_sample);
-    READ_THUMB_NB(file, width);
-    READ_THUMB_NB(file, height);
-    READ_THUMB_NB(file, rowstride);
-    READ_THUMB_NB(file, pixel_length);
+    READ_THUMB_NB(source, bits_per_sample);
+    READ_THUMB_NB(source, width);
+    READ_THUMB_NB(source, height);
+    READ_THUMB_NB(source, rowstride);
+    READ_THUMB_NB(source, pixel_length);
 
-    pixel = g_new(guchar, pixel_length);
-    if (fread(pixel, 1, pixel_length, file) != pixel_length) {
-        g_free(pixel);
+    pixel = read_buffer(source, pixel_length, FALSE);
+    if (pixel == NULL)
         return FALSE;
-    }
 
     *thumb = gdk_pixbuf_new_from_data(pixel, colorspace, has_alpha,
                                       bits_per_sample, width, height, rowstride,
-                                      (GdkPixbufDestroyNotify) g_free, NULL);
+                                      destroy_func(source), NULL);
 
     if (*thumb == NULL) {
-        g_free(pixel);
+        free_buffer(source, pixel);
         return FALSE;
     }
 
     return TRUE;
 }
 
-static gboolean read_thumb(FILE * file, gchar ** hash_key, GdkPixbuf ** thumb)
+static gboolean read_thumb(struct coll_src *source, gchar ** hash_key,
+                           GdkPixbuf ** thumb)
 {
     gint has_thumb, hash_key_length;
 
-    has_thumb = read_number(file);
+    has_thumb = read_number(source);
     if (has_thumb == 0)
         return TRUE;
 
     if (has_thumb != 1)
         return FALSE;
 
-    hash_key_length = read_number(file);
+    hash_key_length = read_number(source);
     if (hash_key_length <= 0)
         return FALSE;
 
     hash_key_length++;
-    *hash_key = g_new(gchar, hash_key_length);
-
-    if (fread(*hash_key, 1, hash_key_length, file) != hash_key_length ||
-        (*hash_key)[hash_key_length - 1] != '\0') {
-
-        g_free(*hash_key);
+    *hash_key = read_buffer(source, hash_key_length, TRUE);
+    if (*hash_key == NULL)
         return FALSE;
-    }
 
-    return read_pixbuf(file, thumb);
+    return read_pixbuf(source, thumb);
 }
 
-static tree_item *read_entry(FILE * file)
+static tree_item *read_entry(struct coll_src *source)
 {
     tree_item *item;
     gint len;
     gchar *path, *hash_key = NULL;
     GdkPixbuf *thumb = NULL;
 
-    if (check_magic("<FILE>", file) == FALSE)
+    if (check_magic(source, "<FILE>") == FALSE)
         return NULL;
 
-    len = read_number(file);
+    len = read_number(source);
     if (len <= 0)
         return NULL;
 
     len++;
-    path = g_new(gchar, len);
-    if (fread(path, 1, len, file) != len || path[len - 1] != '\0') {
-        g_free(path);
+    path = read_buffer(source, len, TRUE);
+    if (path == NULL)
+        return NULL;
+
+    if (read_thumb(source, &hash_key, &thumb) == FALSE) {
+        free_buffer(source, path);
         return NULL;
     }
 
-    if (read_thumb(file, &hash_key, &thumb) == FALSE) {
-        g_free(path);
-        return NULL;
-    }
-
-    if (check_magic("</FILE>", file) == FALSE) {
-        g_free(path);
-        g_free(hash_key);
+    if (check_magic(source, "</FILE>") == FALSE) {
+        free_buffer(source, path);
+        free_buffer(source, hash_key);
         if (hash_key != NULL)
             g_object_unref(thumb);
 
@@ -697,13 +834,55 @@ static tree_item *read_entry(FILE * file)
     return item;
 }
 
-#define CHECK_MAGIC(file, str)                     \
-    do {                                           \
-        gboolean res = check_magic((str), (file)); \
-        if (res == FALSE)                          \
-            goto error;                            \
+#define CHECK_MAGIC(source, str)                 \
+    do {                                         \
+        gboolean res = check_magic(source, str); \
+        if (res == FALSE)                        \
+            goto error;                          \
     } while (0)
 
+static struct coll_src *build_source(FILE * file)
+{
+    off_t length;
+    struct coll_src *source = g_new(struct coll_src, 1);
+
+    if (fseeko(file, 0, SEEK_END) < 0)
+        goto no_mmap;
+
+    length = ftello(file);
+    if (length < 0) {
+        perror("ftello");
+        goto no_mmap;
+    }
+
+    source->base = mmap(NULL, length, PROT_READ, MAP_PRIVATE, fileno(file), 0);
+    if (source->base == MAP_FAILED) {
+        perror("mmap");
+        goto no_mmap;
+    }
+
+    source->is_file = FALSE;
+    source->ptr = source->base;
+    source->end = source->base + length;
+    goto ok;
+
+  no_mmap:
+    fseeko(file, 0, SEEK_SET);
+
+    source->is_file = TRUE;
+    source->file = file;
+
+  ok:
+    return source;
+}
+
+static void destroy_source(struct coll_src *source, gboolean ok)
+{
+    if (ok == FALSE && source->is_file == FALSE)
+        if (munmap(source->base, source->end - source->base) < 0)
+            perror("munmap");
+}
+
 gint load_dot_gliv_from_file(FILE * file, gboolean in_the_head)
 {
     gint count, i = 0;
@@ -711,11 +890,14 @@ gint load_dot_gliv_from_file(FILE * file, gboolean in_the_head)
     gchar **filenames = NULL;
     gboolean ok = TRUE, cancel = FALSE;
     gint nb_inserted = -1;
+    struct coll_src *source;
 
-    CHECK_MAGIC(file, "GLiv <COLLECTION>");
-    CHECK_MAGIC(file, "1");
+    source = build_source(file);
 
-    count = read_number(file);
+    CHECK_MAGIC(source, "GLiv <COLLECTION>");
+    CHECK_MAGIC(source, "1");
+
+    count = read_number(source);
     if (count < 0)
         goto error;
 
@@ -725,7 +907,7 @@ gint load_dot_gliv_from_file(FILE * file, gboolean in_the_head)
     for (i = 0; i < count; i++) {
         tree_item *item;
 
-        item = read_entry(file);
+        item = read_entry(source);
         if (item == NULL)
             goto error;
 
@@ -739,7 +921,7 @@ gint load_dot_gliv_from_file(FILE * file, gboolean in_the_head)
         }
     }
 
-    CHECK_MAGIC(file, "</COLLECTION>");
+    CHECK_MAGIC(source, "</COLLECTION>");
 
     update_progress(NULL, count - 1, count);
     nb_inserted = 0;
@@ -767,11 +949,12 @@ gint load_dot_gliv_from_file(FILE * file, gboolean in_the_head)
     ok = FALSE;
 
   ok:
+
     while (i) {
         i--;
-        g_free(items[i]->path);
+        free_buffer(source, items[i]->path);
         if (ok == FALSE && items[i]->thumb_key != NULL) {
-            g_free(items[i]->thumb_key);
+            free_buffer(source, items[i]->thumb_key);
             g_object_unref(items[i]->thumb);
         }
         g_free(items[i]);
@@ -779,6 +962,7 @@ gint load_dot_gliv_from_file(FILE * file, gboolean in_the_head)
     g_free(items);
     g_free(filenames);
     destroy_progress_dialog();
+    destroy_source(source, ok);
     return nb_inserted;
 }
 
@@ -814,14 +998,15 @@ gint load_dot_gliv(const gchar * filename, gboolean in_the_head)
 
 gboolean deserialize_collection(void)
 {
-    const gchar *filename;
+    gchar *filename;
     gint nb_inserted;
 
-    filename = get_a_file(_("Choose a collection to load"));
+    filename = get_collection_source();
     if (filename == NULL)
         return FALSE;
 
     nb_inserted = load_dot_gliv(filename, FALSE);
+    g_free(filename);
     if (nb_inserted <= 0)
         return FALSE;
 
