@@ -10,6 +10,6 @@
 #define MIPMAP_RATIO     0.75
 
 /* Font for the help box. */
-#define FONT "fixed Medium 10"
+#define FONT "Mono 10"
 
 #endif
