@@ -5,7 +5,7 @@
 #include "gliv-image.h"
 
 void resize_window(gint width, gint height, gboolean force);
-void show_dialog(GtkWidget * widget, const gchar * name);
+void show_message(GtkWidget * widget, const gchar * name);
 gint run_modal_dialog(GtkDialog * dialog);
 void set_loading_entry(const gchar * filename);
 gboolean toggle_widget(GtkWidget * widget, gboolean * flag);
