@@ -77,15 +77,20 @@ void install_segv_handler(void)
 
 void fill_ident(GlivImage * im)
 {
-    gchar *alpha_str;
+    gchar *alpha_str = NULL;
 
     g_free(im->ident);
 
-    alpha_str = im->has_alpha ? _("alpha channel") : _("no alpha channel");
+    if (im->has_alpha)
+        alpha_str = g_strconcat(" (", _("alpha channel"), ")", NULL);
+    else
+        alpha_str = g_strdup("");
 
-    im->ident = g_strdup_printf("%s (%u/%u) (%s)",
+    im->ident = g_strdup_printf("%s (%u/%u)%s",
                                 filename_to_utf8(im->node->data),
                                 im->number + 1, get_list_length(), alpha_str);
+
+    g_free(alpha_str);
 }
 
 const gchar *get_extension(const gchar * filename)
