@@ -415,7 +415,7 @@ void configure_matrix(GlivImage * im)
     if (options->maximize || options->scaledown)
         matrix_set_max_zoom(-1, -1, TRUE);
 
-    else if (options->no_center)
+    if (options->no_center)
         matrix_set_top_left();
 
     current_image = old;
