@@ -40,6 +40,7 @@
 #include "rcfile.h"
 #include "mnemonics.h"
 #include "tree.h"
+#include "images_menus.h"
 
 extern rt_struct *rt;
 extern options_struct *options;
@@ -117,11 +118,12 @@ static void apply(void)
             start_slide_show();
         }
 
-    if (options->thumbnails != new_options->thumbnails ||
-        options->thumb_width != new_options->thumb_width ||
-        options->thumb_height != new_options->thumb_height ||
-        options->mnemonics != new_options->mnemonics)
-        invalidate_last_tree();
+    if (is_rebuilding_a_menu() == FALSE)
+	if (options->thumbnails != new_options->thumbnails ||
+	    options->thumb_width != new_options->thumb_width ||
+	    options->thumb_height != new_options->thumb_height ||
+	    options->mnemonics != new_options->mnemonics)
+	    invalidate_last_tree();
 
     bg_color_changed = !gdk_color_equal(&options->bg_col, &new_options->bg_col);
 
@@ -195,6 +197,22 @@ static gboolean on_save_now_clicked(void)
     return FALSE;
 }
 
+/* Transitions */
+
+static gboolean on_transitions_toggled(GtkTable * transitions_table,
+				       GtkToggleButton * button)
+{
+    gboolean activated = gtk_toggle_button_get_active(button);
+
+    gtk_widget_set_sensitive(GTK_WIDGET(transitions_table), activated);
+    return FALSE;
+}
+
+static gboolean on_transition_table_realize(GtkWidget * transitions_table) {
+    gtk_widget_set_sensitive(transitions_table, options->transitions);
+    return FALSE;
+}
+
 #include "glade_options.c"
 
 gboolean show_options(void)
