@@ -27,12 +27,9 @@
 #include <string.h>             /* strlen(), memcpy() */
 #include <fcntl.h>              /* open(), O_RDONLY, close(), lseek() */
 
-#ifndef HAVE_FTELLO
-#define ftello ftell
-#endif
-
 #include "gliv.h"
 #include "rcfile.h"
+#include "large_files.h"
 #include "options.h"
 #include "messages.h"
 #include "actions.h"
@@ -70,8 +67,9 @@ static options_struct opts = {
     .no_center = FALSE,
     .confirm_quit = TRUE,
     .save_quit = TRUE,
+    .transitions = TRUE,
     .duration = 10,
-    .fps = -1,
+    .fps = 100,
     .thumb_width = 128,
     .thumb_height = 64,
     .notice_time = 500,
@@ -113,6 +111,7 @@ static option_struct option_names[] = {
 { "no_center",    &opts.no_center,    N_("No automatic image centering"),   1 },
 { "confirm_quit", &opts.confirm_quit, N_("Confirm before quitting"),        1 },
 { "save_quit",    &opts.save_quit,    N_("Save options when quitting"),     1 },
+{ "transitions",  &opts.transitions,  N_("Enable transitions"),             1 },
 { "fps",          &opts.fps,          N_("Maximum framerate"),              0 },
 { "delay",        &opts.delay,        N_("Delay before hiding the cursor"), 0 },
 { "history",      &opts.history_size, N_("History length"),                 0 },
@@ -243,7 +242,7 @@ static void foreach_rcfile(rcfile_func func)
 {
     gchar *system_wide[] = { "/etc/glivrc", SYSCONFDIR "/glivrc" };
     gchar *filename;
-    int i;
+    gint i;
 
     for (i = 0; i < G_N_ELEMENTS(system_wide); i++) {
         filename = system_wide[i];
