@@ -22,6 +22,7 @@
  * Rendering functions *
  ***********************/
 
+#include <sys/time.h>           /* struct timeval, gettimeofday() */
 #include <gtk/gtkgl.h>
 
 #include "gliv.h"
@@ -52,6 +53,9 @@ extern GtkWidget *gl_widget;
 /* The gtk timer that calls redraw(). */
 static guint idle_id = 0;
 
+/* When did the last redraw occured, used when limiting the fps */
+static struct timeval last_redraw;
+
 static void set_filter(gint filter)
 {
     gint id;
@@ -202,8 +206,8 @@ static void display_last_image_notice(void)
     const gchar *msg;
     PangoContext *pc;
     PangoLayout *pl;
-    int x = 10, y = 10, dx = 5, dy = 5;
-    int w, h;
+    gint x = 10, y = 10, dx = 5, dy = 5;
+    gint w, h;
 
     msg = get_image_notice();
     if (msg == NULL)
@@ -261,6 +265,9 @@ void draw_current_image(void)
 
     if (alpha_checks)
         glPopAttrib();
+
+    if (options->fps > 0)
+        gettimeofday(&last_redraw, NULL);
 }
 
 /* Called by the timer when a redraw is needed. */
@@ -364,6 +371,25 @@ void zoom_in(gfloat ratio)
     refresh(REFRESH_IMAGE | REFRESH_STATUS | APPEND_HISTORY);
 }
 
+static gint get_fps(void)
+{
+    struct timeval now;
+    gint diff;
+
+    gettimeofday(&now, NULL);
+    if (now.tv_sec - last_redraw.tv_sec > 2)
+        return 0;
+
+    diff = (now.tv_sec - last_redraw.tv_sec) * G_USEC_PER_SEC +
+        now.tv_usec - last_redraw.tv_usec;
+
+    if (diff == 0)
+        return G_USEC_PER_SEC;
+
+    return G_USEC_PER_SEC / diff;
+}
+
+
 static gboolean refresh_image(void)
 {
     idle_id = 0;
@@ -384,10 +410,24 @@ void refresh(gint what)
         if (options->fps <= 0)
             refresh(REFRESH_IMAGE);
 
-        else if (idle_id == 0)
-            idle_id = gtk_timeout_add(1000 / options->fps,
-                                      (GtkFunction) refresh_image, NULL);
+        else if (idle_id == 0) {
+            gint fps = get_fps();
 
+            if (fps <= 0)
+                refresh(REFRESH_IMAGE);
+
+            else {
+                gint diff = G_USEC_PER_SEC / options->fps - G_USEC_PER_SEC / fps;
+                diff /= 1000;
+
+                if (diff > 0)
+                    idle_id = gtk_timeout_add(diff,
+                                              (GtkFunction) refresh_image,
+                                              NULL);
+                else
+                    refresh(REFRESH_IMAGE);
+            }
+        }
     } else if (what & REFRESH_NOW) {
         if (idle_id != 0)
             g_source_remove(idle_id);
