@@ -88,7 +88,7 @@ void resize_window(gint width, gint height, gboolean force)
     gtk_widget_set_size_request(gl_widget, 1, 1);
 }
 
-void show_dialog(GtkWidget * widget, const gchar * name)
+void show_message(GtkWidget * widget, const gchar * name)
 {
     /* In windowed mode we let the WM choose the position. */
     if (options->fullscreen)
@@ -104,6 +104,10 @@ gint run_modal_dialog(GtkDialog * dialog)
 {
     gint response;
 
+    /* In windowed mode we let the WM choose the position. */
+    if (options->fullscreen)
+        gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
+
     /* We want the cursor to be visible when the dialog is shown. */
     set_hide_cursor_enabled(FALSE);
     response = gtk_dialog_run(GTK_DIALOG(dialog));
